import pytest
import io
from awx_junit import awx_junit
import xml.etree.ElementTree as ET


@pytest.fixture
def awx_sample_output_single_entry():
    with open("tests/fixtures/awx_test_output_small.json", "r") as f:
        return io.StringIO(f.read())


@pytest.fixture
def awx_sample_output():
    with open("tests/fixtures/awx_test_output_full.json", "r") as f:
        return io.StringIO(f.read())


def test_generate_report(awx_sample_output_single_entry, capsys, monkeypatch):
    monkeypatch.setattr("sys.argv", ["pytest"])
    awx_junit.main(awx_sample_output_single_entry)
    captured = capsys.readouterr()
    tree = ET.fromstring(captured.out)
    assert tree.tag == "testsuites"


def test_xml_output_single_entry(awx_sample_output_single_entry, capsys, monkeypatch):
    monkeypatch.setattr("sys.argv", ["pytest"])
    awx_junit.main(awx_sample_output_single_entry)
    captured = capsys.readouterr()
    tree = ET.fromstring(captured.out)
    assert tree.tag == "testsuites"
    assert sum(1 for e in tree.iter("testsuite")) == 1
    assert tree[0].attrib["tests"] == "1"
    assert tree[0].attrib["name"] == "Do stuff task"
    assert tree[0].attrib["time"] == "1.10799"
    assert tree[0].attrib["failures"] == "0"
    assert tree[0].attrib["errors"] == "0"
    assert tree[0].attrib["skipped"] == "0"


def test_xml_output_multiple_entries(awx_sample_output, capsys, monkeypatch):
    monkeypatch.setattr("sys.argv", ["pytest"])
    awx_junit.main(awx_sample_output)
    captured = capsys.readouterr()
    tree = ET.fromstring(captured.out)
    assert tree.tag == "testsuites"
    assert sum(1 for e in tree.iter("testsuite")) == 4
    assert sum(1 for e in tree.iter("failure")) == 1
    assert sum(1 for e in tree.iter("skipped")) == 2
    assert tree.attrib["tests"] == "8"
    assert tree.attrib["failures"] == "1"
    assert tree.find("./testsuite[@name='Verify foo']").attrib["failures"] == "1"
    assert tree.find("./testsuite[@name='Check foobar task']").attrib["skipped"] == "2"
    assert tree.find("./testsuite[@name='Gathering Facts']").attrib["tests"] == "4"
    assert (
        tree.find("./testsuite[@name='Gathering Facts']").attrib["time"] == "7.417543"
    )


def test_pretty_print(awx_sample_output, capsys, monkeypatch):
    monkeypatch.setattr("sys.argv", ["pytest", "-p"])
    awx_junit.main(awx_sample_output)
    captured = capsys.readouterr()
    tree = ET.fromstring(captured.out)
    assert tree.tag == "testsuites"


def test_not_json(awx_sample_output, capsys, monkeypatch):
    monkeypatch.setattr("sys.argv", ["pytest"])
    with pytest.raises(SystemExit) as pytest_wrapped_e:
        awx_junit.main(io.StringIO("foobar"))
    captured = capsys.readouterr()
    assert captured.err.startswith("Input is not a JSON")
    assert pytest_wrapped_e.type == SystemExit
    assert pytest_wrapped_e.value.code == 1



#!/usr/bin/env python3
import argparse
import io
import json
import os
import sys
from collections import defaultdict

from junit_xml import TestCase, TestSuite, to_xml_report_string

AWX_FAILED_EVENTS = ["runner_on_failed", "runner_on_async_failed"]

AWX_OK_EVENTS = ["runner_on_ok", "runner_on_async_ok"]

AWX_ERROR_EVENTS = ["runner_on_error", "runner_on_unreachable"]

AWX_SKIPPED_EVENTS = ["runner_on_skipped"]

AWX_EVENTS = AWX_FAILED_EVENTS + AWX_OK_EVENTS + AWX_ERROR_EVENTS + AWX_SKIPPED_EVENTS


def main(stdin: io.TextIOBase = sys.stdin):
    my_parser = argparse.ArgumentParser(
        prog="awx-junit",
        usage="awx job_events list --job <job_id> --all | awx-junit",
        description=(
            "Generates a JUnit XML report from " "a JSON formatted AWX job events."
        ),
    )
    my_parser.add_argument(
        "-p",
        "--pretty",
        help="Return a pretty-printed version of the document.",
        action="store_true",
    )
    args = my_parser.parse_args()

    if stdin.isatty():
        print("stdin does not contain a JSON", file=sys.stderr)
        my_parser.print_help(sys.stderr)
        sys.exit(1)

    try:
        awx_data = json.load(stdin)
    except TypeError:
        print("Invalid JSON", file=sys.stderr)
        my_parser.print_help(sys.stderr)
        sys.exit(1)
    except ValueError:
        print("Input is not a JSON", file=sys.stderr)
        my_parser.print_help(sys.stderr)
        sys.exit(1)

    # Check datastructure
    if "results" not in awx_data:
        print("Content does not look like AWX job_events payload", file=sys.stderr)
        my_parser.print_help(sys.stderr)
        sys.exit(1)

    test_suites = defaultdict(dict)
    for result in awx_data["results"]:
        if result["event"] in AWX_EVENTS:
            case = TestCase(
                name="{} on {}".format(result["task"], result["host_name"]),
                classname=os.path.splitext(result["event_data"]["playbook"])[0].replace(
                    "/", "."
                ),
                file=result["event_data"]["playbook"],
            )
            if "duration" in result["event_data"]:
                case.elapsed_sec = result["event_data"]["duration"]

            if "end" in result["event_data"]:
                case.timestamp = result["event_data"]["end"]

            # Add failure info
            if result["event"] in AWX_FAILED_EVENTS:
                case.add_failure_info(
                    message=result["event_display"], output=result["stdout"]
                )
            # Add error info
            if result["event"] in AWX_ERROR_EVENTS:
                case.add_error_info(
                    message=result["event_display"], output=result["stdout"]
                )
            # Add Skipped message
            if result["event"] in AWX_SKIPPED_EVENTS:
                case.add_skipped_info(
                    message=result["event_display"], output=result["stdout"]
                )

            # Add TestCase to list
            if result["event_data"]["task_uuid"] in test_suites:
                (test_suites[result["event_data"]["task_uuid"]]["cases"]).append(case)
            else:
                (test_suites[result["event_data"]["task_uuid"]]["name"]) = result[
                    "event_data"
                ]["task"]
                (test_suites[result["event_data"]["task_uuid"]]["cases"]) = [case]
    # Create TestSuites and add to list
    junit_data = []
    for suite in test_suites.keys():
        junit_data.append(
            TestSuite(test_suites[suite]["name"], test_suites[suite]["cases"])
        )
    # Export to JUnit XML
    sys.stdout.write(to_xml_report_string(junit_data, args.pretty))


if __name__ == "__main__":
    main(sys.stdin)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="awx-junit",
    version="1.1",
    entry_points={"console_scripts": ["awx-junit=awx_junit.awx_junit:main"]},
    author="Thushjandan Ponnudurai",
    author_email="thushjandan@gmail.com",
    description=(
        "A Ansible Tower or AWX utility " "to generate a JUnit XML report from a job."
    ),
    long_description=long_description,
    long_description_content_type="text/markdown",
    license="MIT",
    url="https://github.com/thushjandan/awx-junit",
    packages=setuptools.find_packages(),
    install_requires=[
        "junit-xml",
    ],
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Topic :: Software Development :: Build Tools",
        "Topic :: Software Development :: Testing",
    ],
)
